<?php

/*
============================================================================================================
        Hubfinder
============================================================================================================

        Author               : Mike Thomas

        Version              : 3.0

        Description          : Hubfinder

        Copyright            : (c) SeoBook.com, licensed under the GPL ( http://www.gnu.org/licenses/gpl.txt )

        History              : 08-01-05
                               - Initial Write
                             : 08-23-05
                               - Added ability to use google as well as yahoo to find top search results for
                                 cross-referencing.
                               - Added ability to highlight a column for easier viewing.

        Function             : Tool allows webmasters to enter a list of sites relating to a certain topic
                               and then find sites which link to these sites. The end goal being to find sites
                               which function as a "hub" for your topic and who would most likely be willing
                               to link to you.
============================================================================================================
*/

// Uses the nusoap library to ease the process of interfacing with the google api with php.
require_once("nusoap.php");

// Search api keys
$appid = 'Hub-Finder'; // Yahoo
if (!$googlekey) $googlekey = "dsLO4qlQFHKHpJ3N/twAQKcEVccH6Z0o"; // Google

// Configuration
if (!$min_match) $min_match = 2;
if (!$search_num) $search_num = 5;
$max_depth = 5; // number of queries to be performed per site (50 results per query)

// Close out the page
function done() {
  echo '</form></body></html>';
  exit;
}

function partial_in_array($key,$array) {
  foreach($array AS $element) {
    if (is_numeric(strpos($element,$key))) {
      return $element;
    }
  }
  return;
}

// Function to convert the XML into an array
function xml_to_result($dom) {
  global $totalResultsReturned;

  $root = $dom->document_element();
  $res['totalResultsReturned'] = $root->get_attribute('totalResultsReturned');

  $i = 0;

  $node = $root->first_child();
  while($node) {
    switch($node->tagname) {
      case 'Result':
        $subnode = $node->first_child();
        while($subnode) {
          $subnodes = $subnode->child_nodes();
          if(!empty($subnodes)) foreach($subnodes as $k=>$n) {
            if(empty($n->tagname)) $res[$i][$subnode->tagname] = trim($n->get_content());
            else $res[$i][$subnode->tagname][$n->tagname]=trim($n->get_content());
          }
          $subnode = $subnode->next_sibling();
        }
        break;
      default:
        $res[$node->tagname] = trim($node->get_content());
        $i--;
        break;
    }
    $i++;
    $node = $node->next_sibling();
  }
  return $res;
}

// Function to display links with all options and information
function show_links ($final_output) {
  global $all_sites, $min_match;
  echo "<tr style=\"font-weight: bold;\">\n";
  for ($y = 0; $y < count($all_sites); $y++) {
    $temp = $y + 1;
    echo "<td align=\"center\">";
    echo "<a href=\"#\" onClick=\"highlightColumn('col$y')\">" . $temp . "</a>";
    echo "</td>\n";
  }
  echo "<td align=\"center\">Site Name</td>\n</tr>";
  for ($x = 0; $x < count($final_output); $x++) {
    if ($final_output[$x]['total'] >= $min_match) {
      echo "<tr>\n";

      $domain = $final_output[$x]['name'];

      for ($y = 0; $y < count($all_sites); $y++) {
        echo "<td align=\"center\" class=\"col$y\">";
        if ($final_output[$x][$y] != '') {
          echo "<a href=\"" . $final_output[$x][$y] . "\">X</a>\n";
        }
        echo "</td>\n";
      }
      echo "<td>";
      echo "[<a href=\"http://www.whois.sc/$domain\">W</a>] ";
      echo "[<a href=\"http://web.archive.org/web/*/http://$domain\">A</a>] ";
      echo "[<a href=\"http://www.google.com/search?q=cache:http://$domain&strip=1\">H</a>] ";
      echo "[<a href=\"http://www.whois.sc/dmoz/$domain\">D</a>] ";
      echo "[<a href=\"http://www.whois.sc/yahoo/$domain\">Y</a>] ";

      echo "<a href=\"http://" . $domain . "\">" . $domain . "</a> ";
      echo "(" . gethostbyname($domain) . ")</td>\n";
      echo "</tr>\n";
    }
  }
}

// Function to display csv
function show_csv ($final_output) {
  global $all_sites, $min_match;
  for ($y = 0; $y < count($all_sites); $y++) {
    echo $y+1;
    echo ",";
  }
  echo "Site Name,IP Address,";
  for ($y = 0; $y < count($all_sites); $y++) {
    echo $y+1;
    echo ",";
  }
  echo "\n";
  for ($x = 0; $x < count($final_output); $x++) {
    if ($final_output[$x]['total'] >= $min_match) {
      $domain = $final_output[$x]['name'];

      for ($y = 0; $y < count($all_sites); $y++) {
        if ($final_output[$x][$y] != '') {
          echo "X";
        }
        echo ",";
      }
      echo "$domain,";
      echo gethostbyname($domain) . ",";
      for ($y = 0; $y < count($all_sites); $y++) {
        if ($final_output[$x][$y] != '') {
          echo $final_output[$x][$y];
        }
        echo ",";
      }
      echo "\n";
    }
  }
}

?>
<!doctype html public "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title>Yahoo Hubfinder</title>
<link rel="stylesheet" href="style.css">
</head>
<body>

  <script type="text/javascript">
function highlightColumn(colname) {
  tmp = document.getElementsByTagName('td');
  for (i = 0; i < tmp.length; i++) {
    if (tmp[i].className == colname) {
      tmp[i].style.backgroundColor = '#FFFF00';
    } else {
      tmp[i].style.backgroundColor = '';
    }
  }
}
</script>
<p>Hub Finder is a free link analysis tool created by <a href="http://www.seobook.com/">www.SeoBook.com</a>. Hub Finder looks for sites which have co occuring links to related authoritative websites on a particular topic. </p>
<ul>
  <li>You can manually enter sites you want to cross check, or grab the top ranked results from Yahoo! Search or Google. </li>
  <li>The depth limit acts as a noise filter since Yahoo! places some of the better backlinks near the top of their results.</li>
  <li>Hub pages which link to related resources may provide powerful links which are given a relevancy boost in community, topic, or authority based search algorithms such as <a href="http://www.seobook.com/archives/000661.shtml">TrustRank</a>, <a href="http://www.searchguild.com/topic_distillation.pdf">Teoma's Topic Distilation [PDF]</a>, <a href="http://www.seobook.com/archives/000626.shtml">Hilltop</a>, and <a href="http://www.seoresearchlabs.com/seo-research-labs-google-report.pdf">Topic Sensitive PageRank [PDF]</a>. </li>
  <li><strong>403 errors</strong> means the API queries are used up for the day. If you are a heavy user please consider mirroring the tool. </li>
  <li>Mirrors: 
  <a href="http://webseodesign.com/seo-tool-chest/hub-finder.php">Web SEO Design</a>,  <a href="http://www.emeraldcoastentrepreneur.com/Hubfinder/hubfinder.php">Emerald Coast Entrepreneur</a>, &amp; 
  <a href="http://www.cyclelicio.us/hubfinder/">Cyclicious</a> </li>
  <li>Questions or feedback? <a href="http://www.seobook.com/archives/001124.shtml">click here</a>. Want the source code and / or background information? <a href="http://tools.seobook.com/hub-finder/">click here</a>. </li>
</ul>
<p>&nbsp; </p>
<form action="hubfinder.php" method="GET">
<table border="0" width="600" cellspacing="0" cellpadding="3">
    <tr>
      <td>
        Subject:<br>
        <input type="text" name="query" value="<?php echo $_GET['query']; ?>">
      </td>
      <td>
        Results:<br>
        <select name="search_num">
          <?php
          for ($x = 1; $x <= 10; $x++) {
            if ($search_num == $x) {
              echo "<option value=\"$x\" selected>$x</option>\n";
            } else {
              echo "<option value=\"$x\">$x</option>\n";
            }
          }
          ?>
        </select>
      </td>
      <td>
        API:<br>
        <select name="whichengine">
          <?php if ($_GET['whichengine'] == 'both') { ?>
            <option value="yahoo">Yahoo!</option>
            <option value="google">Google</option>
            <option value="both" selected>Both</option>
          <?php } elseif ($_GET['whichengine'] == 'google') { ?>
            <option value="yahoo">Yahoo!</option>
            <option value="google" selected>Google</option>
            <option value="both">Both</option>
          <?php } else { ?>
            <option value="yahoo" selected>Yahoo!</option>
            <option value="google">Google</option>
            <option value="both">Both</option>
          <?php } ?>
        </select>
      </td>
      <td>
        Min Match:<br>
        <select name="min_match">
          <?php
          for ($x = 1; $x <= 10; $x++) {
            if ($min_match == $x) {
              echo "<option value=\"$x\" selected>$x</option>\n";
            } else {
              echo "<option value=\"$x\">$x</option>\n";
            }
          }
          ?>
        </select>
      </td>
      <td>
        Depth:<br>
        <select name="depth">
      <?php
      for($i = 1; $i <= $max_depth; $i++) {
        $selected = '';
        $totaldepth = $i * 50;
        if ($i == $depth) $selected = 'selected';
        echo "<option value=\"$i\" $selected>$totaldepth</option>\n";
      }
      ?>
        </select>
        &nbsp;&nbsp;&nbsp;&nbsp;<input type="button" value="Download CSV" onClick="document.forms['csv'].submit()">
      </td>
    </tr>
    <tr>
      <td><a href="http://www.google.com/apis/">Google Key</a>:<br><input type="text" name="googlekey" value="<?php echo $_GET['googlekey']; ?>"></td>
    </tr>
</table>
  <table border="0" width="500" cellspacing="0" cellpadding="3">
    <tr>
      <td bgcolor="#F5F5FF" style="border-top: 1px solid #E0E0EF; border-left: 1px solid #E0E0EF; border-bottom: 1px solid #E0E0EF;">
        Include this site (optional):<br>
        <input type="text" name="individual" value="<?php echo $_GET['individual']; ?>">
      </td>
      <td bgcolor="#F5F5FF" bgcolor="#F5F5FF" style="border-top: 1px solid #E0E0EF; border-bottom: 1px solid #E0E0EF;">
        Link Type:<br>
        <select name="linktype">
          <?php if ($_GET['linktype'] == 'link') { ?>
            <option value="domain">Domain</option>
            <option value="link" selected>Page</option>
          <?php } else { ?>
            <option value="domain" selected>Domain</option>
            <option value="link">Page</option>
          <?php } ?>
        </select>
      </td>
      <td valign="bottom" bgcolor="#F5F5FF" style="border-top: 1px solid #E0E0EF; border-right: 1px solid #E0E0EF; border-bottom: 1px solid #E0E0EF;">
        Application:<br>
        <select name="apply">
          <?php if ($_GET['apply'] == 'mustlink') { ?>
            <option value="sortby">Sort By</option>
            <option value="mustlink" selected>Must Link</option>
          <?php } else { ?>
            <option value="sortby" selected>Sort By</option>
            <option value="mustlink">Must Link</option>
          <?php } ?>
        </select>
      </td>
      <td valign="bottom">
        <input type="submit" value="Query">
        <input type="button" value="Start Over" onClick="document.location='hubfinder.php'">
      </td>
    </tr>
  </table>
<?php
echo "<strong>Enter up to 10 sites:</strong><br><textarea name=\"manual_filter\" rows=5 cols=60 wrap=\"off\">$manual_filter</textarea></form>\n";

// If they haven't submitted a query, end the page.
if (!isset($query)) done();

// Break up the sites entered by the user into an array
$manual_sites = explode("\n",$manual_filter);
$searched_sites = array();

if ($whichengine != 'google') {
  // Construct the query for the Yahoo api making sure to remove results from sites already entered manually
  $query = str_replace(' ','%20',$query);
  $q='http://api.search.yahoo.com/WebSearchService/V1/webSearch?query=' . $query;
  foreach ($manual_sites AS $site) {
    $q .= '%20-site:' . trim($site);
  }
  $q .= '&results=' . $search_num . '&appid=' . $appid;

  // Perform the Yahoo query to get the top x sites for the topic entered
  $site_results = file_get_contents($q);
  $dom = domxml_open_mem($site_results,DOMXML_LOAD_PARSING,$error);
  if ($dom) $res = xml_to_result($dom);

  // Get the Yahoo results in an array
  for ($x = 0; $x < $res[totalResultsReturned]; $x++) {
    $url = $res[$x][Url];
    $url_array = parse_url($url);
    $url = $url_array['host'];
    if (!in_array($url,$searched_sites)) array_push($searched_sites,$url);
  }
}
$maxresults = intval($search_num);
if ($whichengine != 'yahoo') {
  // Get the top search results from google.
  $parameters = array(
              "key"         => $googlekey,
              "q"           => $query,
              "start"       => 0,
              "maxResults"  => $maxresults,
              "filter"      => true,
              "restrict"    => "",
              "safeSearch"  => false,
              "lr"          => "lang_en",
              "ie"          => "",
              "oe"          => ""
              );


  $soapclient = new soapclient("http://api.google.com/search/beta2"); // [1]
  $result = $soapclient->call("doGoogleSearch", $parameters, "urn:GoogleSearch");

  for ($i = 0; $i < $result["endIndex"]; $i++) {
    $element = $result[resultElements][$i];
    $url = $element["URL"];
    $url_array = parse_url($url);
    $url = $url_array['host'];
    if (!in_array($url,$searched_sites)) array_push($searched_sites,$url);
  }
}

$all_results = array();
$site_count = 0;

$all_sites = array();

if ($individual != '') array_push($all_sites,$individual);

foreach ($manual_sites AS $site) {
  if ($site != '') array_push($all_sites,$site);
}
foreach ($searched_sites AS $site) {
  if ($site != '') array_push($all_sites,$site);
}

foreach ($all_sites AS $site) {
  $site_results_array = array();
  $site = trim($site);
  if (strlen($site) > 1) {
    $query = str_replace('http://','',$site);
    $domain = parse_url("http://" . $query);

    for ($i = 0; $i < $depth; $i++) {
      $start = 50 * $i;
      if (($site_count == 0) && ($individual != '') && ($linktype == 'link')) {
        $q='http://api.search.yahoo.com/WebSearchService/V1/webSearch?query=link:http://' . str_replace('http://','',$individual) . "%20-site:" . $domain[host];
      } else {
        $q='http://api.search.yahoo.com/WebSearchService/V1/webSearch?query=linkdomain:' . $domain[host] . "%20-site:" . $domain[host];
      }
      if ($start == 0) {
        $site_results = file_get_contents($q.'&results=50&appid=' . $appid);
      } else {
        $site_results = file_get_contents($q.'&results=50&appid=' . $appid . '&start=' . $start);
      }
      $dom = domxml_open_mem($site_results,DOMXML_LOAD_PARSING,$error);
      if ($dom) $res = xml_to_result($dom);
      for ($x = 0; $x < $res[totalResultsReturned]; $x++) {
        $full_url = $res[$x][Url];
        $url_array = parse_url($full_url);
        $url = $url_array['host'];
        array_push($site_results_array,$full_url);
        if (!in_array($url,$all_results)) {
          array_push($all_results,$url);
        }
      }
    }

    $sites[$site_count] = $site_results_array;
    $site_count++;
  }
}

$final_output = array();
for ($y = 0; $y < count($all_results); $y++) {
  $final_output[$y]['name'] = $all_results[$y];
  for ($x = 0; $x < count($all_sites); $x++) {

    $this_link = partial_in_array($all_results[$y],$sites[$x]);

    if ($this_link != '') {
      if (($x == 0) || ($apply != 'mustlink') || ($individual == '') || ($final_output[$y][0] != '')) {
        $final_output[$y][$x] = $this_link;
        $final_output[$y]['total']++;
      }
    } else {
        $final_output[$y][$x] = '';
    }
  }
}

foreach($final_output as $result) {
  $row_sorting[] = $result['total'];
}
array_multisort($row_sorting, SORT_DESC, $final_output);

if (($individual != '') && ($apply == 'sortby')) {
  foreach($final_output as $result) {
    $row_sorting2[] = $result[0];
  }
  array_multisort($row_sorting2, SORT_DESC, $final_output);
}
?>
<style>
table tr td {
  padding: 3px;
}
</style>
<?php

echo "<div id=\"results\">Querying the following " . count($all_sites) . " sites</div>\n";

for ($y = 1; $y <= count($all_sites); $y++) {
  echo "$y: " . $all_sites[$y-1] . "<br>\n";
}

$num_results = 0;
for ($x = 0; $x < count($final_output); $x++) {
  if ($final_output[$x]['total'] >= $min_match) {
    $num_results++;
  }
}

echo "<div id=\"results\">Showing " . $num_results . " sites with at least " . $min_match . " matching backlinks from " . count($all_results) . " search results</div>\n";
echo "<table id=\"def\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n";

show_links($final_output);

?>
</table>


<div id="getcsv">
<form name="csv" id="csv" action="gencsv.php" method="post">
<textarea name="content" style="display: none" rows="1000" cols="1000">
<?php
echo "No.,Site\n";
for ($y = 1; $y <= count($all_sites); $y++) {
  echo "$y," . $all_sites[$y-1] . "\n";
}
echo "\n\n";
show_csv($final_output);
?> 
</textarea>
</form>
</div>

</body> 
</html> 
