<?php
/*
============================================================================================================
        Link Harvester                                                   Sample File (backlinks.php)
============================================================================================================

        Author               : Mike Thomas

        Version              : 1.0

        Description          : LinkHarvester, a sample implementation of the Yahoo API.

        Copyright            : (c) SeoBook.com, licensed under the GPL ( http://www.gnu.org/licenses/gpl.txt )

        History              : 05-04-05
                               - Initial Write
                             : 05-21-05
                               - Added links to Google cache and sorted out .mil domains.
                             : 07-03-05
                               - Filtered domains automatically drop the www so that it blocks the whole URL. Also allows
                               manual entry of a site to block. This works especially well if people are buying links
                               across 20 regional sites off the main bizjournals domain... now you can just enter the
                               bizjournals domain and block all it's subdomains. Also added links to WhoIs Source pages
                               for Yahoo! Directory and DMOZ listings.
              							 : 11-25-05
              							   - Added MSN backlinks.
              							 : 1-08-06
              							   - Added the function to only check deeplinks pointing at a site. Also added links next to
              							   each result to check all backlinks from that specific domain via Yahoo! or MSN.
              							 : 1-22-06
              							   - Fixed CSV output
              							   - Moved the Download CSV button to line up with the other buttons and changed the button
              							   name to simply 'CSV'
              							   - Fixed the wording in some link titles

        Function             : Tool allows webmasters to enter in the URLs of competing websites and find out the unique
                               number of inbound linking sites, number of inbound linking .edu & .gov sites, and the
                               number of inbound linking C class IP addresses. Change filename to backlinks.php to get it
                               to work.

        More info            : at http://www.seobook.com/archives/000842.shtml
============================================================================================================
*/
require_once 'class_gamy.php';

$gamy = new gamy();
$gamy->yahoo_appid = 'LinkHarvester';

// Configuration
$columns = 2;
$autocheck = 4; // Greater than this number will be auto-checked.

// If there is no depth selected, set it to 5 be default
if ($depth > 5) {
  // Do Nothing
} else {
  $depth = 5;
}
$alreadyfiltered = $depth * 50;
$filteredstart = $alreadyfiltered + 100;

// Close out the page
function done() {
  echo '</form></body></html>';
  exit;
}

// Function to display links with all options and information
function show_links ($links) {
  $count = 0;
  global $columns, $autocheck, $hostcount, $query;
  foreach($links as $host => $info) {
    if (!is_numeric(strpos($info['page']['url'],'http://'))) {
      $url = 'http://' . $info['page']['url'];
    } else {
      $url = $info['page']['url'];
    }
    if ($count % $columns == 0) echo "  <tr>\n";
    $count++;
    $hostcount++;
    if ($info['links'] > $autocheck) { $checked = ' checked'; }
    else { $checked = ''; }
    echo "    <td width=\"50%\"><input type=\"checkbox\" value=\"" . str_replace('www.','',$host) . "\" name=\"x$hostcount\"" . $checked . ">";
    echo "[<a href=\"http://www.whois.sc/$host\" title=\"Search whois.sc for $host\">W</a>] ";
    echo "[<a href=\"http://web.archive.org/web/*/http://$host\" title=\"Search archive.org for $host\">A</a>] ";
    echo "[<a href=\"http://www.google.com/search?q=cache:$url\" title=\"Google's cache of $url\">G</a>] ";
    echo "[<a href=\"http://www.google.com/search?q=cache:$url&strip=1\" title=\"Google's text cache of $url\">T</a>] ";
    echo "[<a href=\"http://www.google.com/search?q=cache:http://$host&strip=1\" title=\"Google's text cache of $host\">H</a>] ";
    echo "[<a href=\"http://www.whois.sc/dmoz/$host\" title=\"DMOZ listing for this site\">D</a>] ";
    echo "[<a href=\"http://www.whois.sc/yahoo/$host\" title=\"Yahoo! listing for this site\">Y</a>] ";
    echo "[<a href=\"http://search.yahoo.com/search?p=site:$host%20+linkdomain:$query\" title=\"Search Yahoo! for pages from $host linking to $query\">S</a>] ";
    echo "[<a href=\"http://search.msn.com/results.aspx?q=site:$host%20+AND+linkdomain:$query\" title=\"Search MSN for pages from $host linking to $query\">M</a>] ";

    if ($info['links'] > $autocheck) {
      echo "<strong><a href=\"$url\">" . $host . "</a> ";
      echo "(" . $info['links'] . ") " . $info['ip'] . "</strong></td>\n";
    }
    else {
      echo "<a href=\"$url\">" . $host . "</a> ";
      echo "(" . $info['links'] . ") " . $info['ip'] . "</td>\n";
    }

    if (($count % $columns == 0) || ($count == count($edu))) echo "  </tr>\n";
  }
}

?>
<!doctype html public "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title>Yahoo Backlink Information</title>
<link rel="stylesheet" href="style.css">
</head>
<body>

<p>Link Harvester is a <strong>free open source link analysis tool</strong> from
  <a href="http://www.seobook.com">www.seobook.com</a>. Link Harvester returns:
</p>
<ul>
  <li>unique linking domains</li>
  <li>their IP addresses &amp; unique c block addresses of the links</li>
  <li>links from .gov, .mil, .edu, or .ac.uk domains. Links from these domains
    are likely rated better in authority type algorithms such as <a href="http://www.seobook.com/archives/000626.shtml">Hilltop</a>
    or <a href="http://www.seobook.com/archives/000661.shtml">TrustRank</a>.</li>
  <li>Link Harvester also bolds and autochecks domains that link from 5 or more
    pages. You can then requery the Yahoo! database and spider deeper than the
    1,000 link limit.</li>
  <li>Link Harvester uses the Yahoo! API, provides CSV outputs, and is an open
    source tool. If you are a heavy user or the query limit (5,000 daily requests)
    is exceeded feel free to host your own copy of Link Harvester on your site.
    Setup should take under a minute.</li>
  <li> Here is the <a href="http://tools.seobook.com/link-harvester/">source code</a>.
    Please note, your host must support DOM XML to host this tool.</li>
  <li>Link Harvester also links to the Internet Archives, WhoIs Source, and Google
    cache near each link. </li>
  <li>Questions, comments, feedback? <a href="http://www.seobook.com/archives/000842.shtml">Let
    me know what you think</a>.</li>
  <li><strong>403 errors</strong> means the tool is used up for the day.</li>
  <li>please try one of the mirrors or mirror the tool on your server if you use
    it frequently</li>
  <li><strong>tool mirrors:</strong> thanks to <a href="http://www.seotips101.com/harvester/">Todd</a>
    , <a href="http://www.seo-scoop.com/linkharvester/backlinks.php">Donna</a>,
    <a href="http://webseodesign.com/seo-tool-chest/backlinks.php">Martin</a>
    &amp; <a href="http://tools.marketingfan.com/Link-Harvester/backlinks.php">Christoph</a></li>
</ul>
    
<form action="backlinks.php" method="GET">
  <table border="0" width="500">
    <tr>
      <td colspan="5">
        URL (ex. www.site.com):<br>
        <input type="text" style="width: 200px;" name="query" value="<?php echo $_GET['query']; ?>">
      </td>
    </td>
    <tr>
      <td>
        Link Type:<br>
        <select name="linktype">
          <?php if ($_GET['linktype'] == 'link') { ?>
            <option value="linkdomain">Domain</option>
            <option value="link" selected>Page</option>
            <option value="deep">Deep Links</option>
          <?php } elseif ($_GET['linktype'] == 'domain') { ?>
            <option value="linkdomain" selected>Domain</option>
            <option value="link">Page</option>
            <option value="deep">Deep Links</option>
          <?php } elseif ($_get['linktype'] == 'deep') { ?>
            <option value="linkdomain">Domain</option>
            <option value="link">Page</option>
            <option value="deep" selected>Deep Links</option>
          <?php } else { ?>
            <option value="linkdomain">Domain</option>
            <option value="link">Page</option>
            <option value="deep">Deep Links</option>
          <?php } ?>
        </select>
      </td>
      <td>
        Query Depth:<br>
        <select name="depth">
      <?php
      for($i = 1; $i <= 4; $i++) {
        $selected = '';
        $totaldepth = $i * 250;
        $thisdepth = $i * 5;
        if ($thisdepth == $depth) $selected = 'selected';
        echo "<option value=\"$thisdepth\" $selected>$totaldepth</option>";
      }
      ?>
        </select>
      </td>
      <td>
        Engine:<br>
        <select name="engine">
          <?php if ($_GET['engine'] == 'msn') { ?>
            <option value="yahoo">Yahoo!</option>
            <option value="msn" selected>MSN</option>
            <option value="both">Both</option>
          <?php } elseif ($_GET['engine'] == 'both') { ?>
            <option value="yahoo">Yahoo!</option>
            <option value="msn">MSN</option>
            <option value="both" selected>Both</option>
          <?php } else { ?>
            <option value="yahoo" selected>Yahoo!</option>
            <option value="msn">MSN</option>
            <option value="both">Both</option>
          <?php } ?>
        </select>
      </td>
      <td valign="bottom">
        <input type="submit" value="Query">
      </td>
      <td valign="bottom">
        <input type="button" value="Start Over" onClick="document.location='backlinks.php'">
      </td>
      <td valign="bottom">
        <input type="button" onClick="forms['csv'].submit();" value="CSV">
      </td>
    </tr>
  </table>
<?php

if (!isset($query)) done();

// Calculate deep link percentage.
$query = str_replace('http://','',$_REQUEST['query']);
$domain = parse_url("http://" . rawurlencode($query));

$gamy->result_num = 1;

if ($engine == 'msn') {

  $gamy->query = 'linkdomain:' . $domain[host];
  $gamy->search_msn();
  $totaldomainresults = $gamy->totals['msn'];

  $gamy->query = 'http://' . $domain[host];
  $gamy->search_msn();
  $totalpagesindexed = $gamy->totals['msn'];

  $gamy->query = 'link:http://' . $domain[host];
  $gamy->search_msn();
  $totalhomeresults = $gamy->totals['msn'];

} else {

  $gamy->query = 'linkdomain:' . $domain[host];
  $gamy->search_yahoo();
  $totaldomainresults = $gamy->totals['yahoo'];

  $gamy->query = 'http://' . $domain[host];
  $gamy->search_yahoo();
  $totalpagesindexed = $gamy->totals['yahoo'];

  $gamy->query = 'link:http://' . $domain[host];
  $gamy->search_yahoo();
  $totalhomeresults = $gamy->totals['yahoo'];

}

if ($linktype == 'link') {
  $gamy->query = 'link:http://' . $domain[host];
} elseif ($linktype == 'domain') {
  $gamy->query = 'linkdomain:' . $domain[host];
} elseif ($linktype == 'deep') {
  $gamy->query = 'linkdomain:' . $domain[host] . ' -link:http://' . $domain[host] . ' -link:http://' . str_replace('www.','',$domain[host]);
}

$filtered_sites = array();
$filtered_cblocks = array();
for($i = 0; $i < $filteredstart; $i++) {
  if (isset($_GET['x'.$i])) {
    $site = str_replace('www.','',$_GET['x'.$i]);
    $gamy->exclude_site($site);
    array_push($filtered_sites,$site);
    $cblock = substr(gethostbyname($site),0,strrpos(gethostbyname($site),'.'));
    $filtered_cblocks[$cblock]++;
  }
}

$manual_sites = explode("\n",$manual_filter);
foreach ($manual_sites AS $site) {
  if (strlen($site) > 1) {
    $gamy->exclude_site($site);
    array_push($filtered_sites,$site);
    $cblock = substr(gethostbyname($site),0,strrpos(gethostbyname($site),'.'));
    $filtered_cblocks[$cblock]++;
  }
}

echo "<br><br>";

$gamy->result_num = $depth * 50;

if ($engine == 'yahoo') {
  $gamy->search_yahoo();
  $totalresults = $gamy->totals['yahoo'];
} elseif ($engine == 'both') {
  $gamy->search_msn();
  $gamy->search_yahoo();
  $totalresults = $gamy->totals['msn'];
  $totalresults += $gamy->totals['yahoo'];
} else {
  $gamy->search_msn();
  $totalresults = $gamy->totals['msn'];
}

foreach($gamy->search_results AS $value) {
  $parsed = parse_url('http://' . $value['url']);
  $host = $parsed['host'];

  if ((substr($host,-4,4) == '.edu') || (substr($host,-7,4) == '.edu') || (substr($host,-6,6) == '.ac.uk')) {
    $edu[$host]['links']++;
    if ($edu[$host]['page'] == '') $edu[$host]['page'] = $value;
    if ($edu[$host]['ip'] == '') $edu[$host]['ip'] = gethostbyname($host);
    $cblock = substr($edu[$host]['ip'],0,strrpos($edu[$host]['ip'],'.'));
    $edu_cblocks[$cblock]++;
  } else if ((substr($host,-4,4) == '.gov') || (substr($host,-7,4) == '.gov') || (substr($host,-4,4) == '.mil') || (substr($host,-7,4) == '.mil')) {
    $gov[$host]['links']++;
    if ($gov[$host]['page'] == '') $gov[$host]['page'] = $value;
    if ($gov[$host]['ip'] == '') $gov[$host]['ip'] = gethostbyname($host);
    $cblock = substr($gov[$host]['ip'],0,strrpos($gov[$host]['ip'],'.'));
    $gov_cblocks[$cblock]++;
  } else {
    $def[$host]['links']++;
    if ($def[$host]['page'] == '') $def[$host]['page'] = $value;
    if ($def[$host]['ip'] == '') $def[$host]['ip'] = gethostbyname($host);
    $cblock = substr($def[$host]['ip'],0,strrpos($def[$host]['ip'],'.'));
    $def_cblocks[$cblock]++;
  }
}

echo "<strong>Enter sitenames you want to filter:</strong><br><textarea name=\"manual_filter\" rows=5 cols=60 wrap=\"off\"></textarea>\n";

echo "<div id=\"results\">Showing ";
echo count($edu) + count($gov) + count($def) . " unique domains from the first $alreadyfiltered results of $totalresults total results</div>\n";

$deeplinks = $totaldomainresults - $totalhomeresults;
if ($totaldomainresults > 0) $deeplinkpercentage = round($deeplinks / $totaldomainresults * 100);
else $deeplinkpercentage = 0;
?>
<table width="500" border="0">
  <tr>
    <td>
<?php
echo "Links To Domain: " . $totaldomainresults . "<br>";
echo "Links To Homepage: " . $totalhomeresults . "<br>";
echo "</td><td>";
echo "Pages Indexed: " . $totalpagesindexed . "<br>";
echo "Deep Link Percentage: " . $deeplinkpercentage . "%<br>";
?>
    </td>
  </tr>
</table>
<br>
<?php
$hostcount = 0;

if (count($filtered_sites) >= 1) {
  $count = 0;
  echo "<strong>" . count($filtered_sites) . " Filtered Sites with " . count($filtered_cblocks) . " Unique C Block Addresses</strong><div id=\"filtered\"><table border=\"0\" width=\"100%\">\n";
  foreach($filtered_sites as $site) {
    if ($count % $columns == 0) echo "  <tr>\n";
    $count++;
    echo "    <td width=\"50%\"><input type=\"checkbox\" value=\"" . str_replace('www.','',$site) . "\" name=\"x$alreadyfiltered\" checked>";
    echo "[<a href=\"http://www.whois.sc/$site\" title=\"Search whois.sc for this site\">W</a>] ";
    echo "[<a href=\"http://web.archive.org/web/*/http://$site\" title=\"Search archive.org for this site\">A</a>] ";
    echo "[<a href=\"http://www.google.com/search?q=cache:http://$site&strip=1\" title=\"Search Google's cache for this site\">H</a>] ";
    echo "[<a href=\"http://www.whois.sc/dmoz/$site\" title=\"DMOZ listing for this site\">D</a>] ";
    echo "[<a href=\"http://www.whois.sc/yahoo/$site\" title=\"Yahoo! listing for this site\">Y</a>] ";
    echo "[<a href=\"http://search.yahoo.com/search?p=site:$site%20+linkdomain:$query\" title=\"Search Yahoo! for pages from $site linking to $query\">S</a>] ";
    echo "[<a href=\"http://search.msn.com/results.aspx?q=site:$site%20+AND+linkdomain:$query\" title=\"Search MSN for pages from $site linking to $query\">M</a>] ";
    echo "<a href=\"http://" . $site . "\">" . $site . "</a> ";
    echo gethostbyname($site) . "</td>\n";
    $alreadyfiltered++;
    if ($count % $columns == 0) echo "  </tr>\n";
  }
  echo "</table></div>\n";
}

if ($edu){
  echo "<strong>" . count($edu) . " Unique Educational Domains (*.edu) with " . count($edu_cblocks) . " Unique C Block Addresses</strong><div id=\"edu\"><table border=\"0\" width=\"100%\">\n";
  show_links($edu);
  echo "</table></div>\n";
}

if ($gov){
  echo "<strong>" . count($gov) . " Unique Goverment Domains (*.gov, *.mil) with " . count($gov_cblocks) . " Unique C Block Addresses</strong><div id=\"gov\"><table border=\"0\" width=\"100%\">\n";
  show_links($gov);
  echo "</table></div>\n";
}

if ($def) {
  echo "<strong>" . count($def) . " Unique Commerical Domains (*.com, *.net, etc) with " . count($def_cblocks) . " Unique C Block Addresses</strong><div id=\"def\"><table border=\"0\" width=\"100%\">\n";
  show_links($def);
  echo "</table></div></form\n";
}

?>
<div id="getcsv">
<form name="csv" id="csv" action="gencsv.php" method="post">
<textarea name="content" style="display: none" rows="1000" cols="1000">
<?php

echo "Site Name, Links To Domain, Links To Homepage, Pages Indexed, Deep Link Percentage\n";
echo $_GET['query'] . ', ' . $totaldomainresults . ", " . $totalhomeresults . ", " . $totalpagesindexed . ", " . $deeplinkpercentage . "\n\n";

echo "Filtered Sites\n\n";
echo "Domain Name, Page Linked From, Number of Links, IP Address\n";
foreach($filtered_sites AS $site) {
  echo "$site, N/A, N/A, " . gethostbyname($site) . "\n";
}
echo "\nUnique Educational Domains, Unique C Block Address\n";
echo count($edu) . ", " . count($edu_cblocks) . "\n";
if ($edu){
  echo "Domain Name, Page Linked From, Number of Links, IP Address\n";
  foreach($edu as $host => $info) {
    echo $host . ", " . str_replace(',','%2C',$info['page']['url']) . ", " . $info['links'] . ", " . $info['ip'] . "\n";
  }
}
echo "\nUnique Government Domains, Unique C Block Address\n";
echo count($gov) . ", " . count($gov_cblocks) . "\n";
if ($gov){
  echo "Domain Name, Page Linked From, Number of Links, IP Address\n";
  foreach($gov as $host => $info) {
    echo $host . ", " . str_replace(',','%2C',$info['page']['url']) . ", " . $info['links'] . ", " . $info['ip'] . "\n";
  }
}
echo "\nUnique Commercial Domains, Unique C Block Address\n";
echo count($def) . ", " . count($def_cblocks) . "\n";
if ($def){
  echo "Domain Name, Page Linked From, Number of Links, IP Address\n";
  foreach($def as $host => $info) {
    echo $host . ", " . str_replace(',','%2C',$info['page']['url']) . ", " . $info['links'] . ", " . $info['ip'] . "\n";
  }
}
?>
</textarea>
</form>
</div>

</body>
</html>